#if 1
#include	<asm/io.h>
#include	<asm/ppcops.h>
#include 	<asm/tw16550.h>

void s403_init(void);
int s403_getchar(void);
void s403_putchar(int);
void s403_print(const char *);
int s403_readline(char *, int);

#define RD_REG(base, offset) inb(base+offset)
#define WR_REG(base, offset, value) outb(value, (int)base+offset)

static inline void SyncIO( void )
{
        __asm__( "eieio " : : );
}
static volatile unsigned char *S403_BASE = (volatile unsigned char *) IOBASE_UART;

int	s403_ready = 0;

void
s403_virtualinit(void) {
	s403_ready = 0;
	S403_BASE = (volatile unsigned char *)IOBASE_UART;
	s403_init();
}

void
s403_realinit(void) {
	s403_ready = 0;
	S403_BASE = (volatile unsigned char *) IOBASE_UART;
	s403_init();
}

#define map_baud(b,s) (s/(b*16))

void
s403_init() {
	if (!s403_ready) {
  		register volatile unsigned char *cntrl = S403_BASE;

		/* Make sure it exists ! */
		WR_REG(cntrl, SCRATCH_PAD_REG, 0x5a);
		SyncIO();
		WR_REG(cntrl, LINE_STATUS_REG, 0);
		SyncIO();
		if (RD_REG(cntrl, SCRATCH_PAD_REG) != 0x5a) {
			s403_ready = -1;
			return;
		}
		WR_REG(cntrl, SCRATCH_PAD_REG, 0xa5);
		SyncIO();
		WR_REG(cntrl, LINE_STATUS_REG,0);
		SyncIO();
		if (RD_REG(cntrl, SCRATCH_PAD_REG) != 0xa5) {
			s403_ready = -1;
			return;
		}
  		/* make sure the transmit FIFO is empty */
  		while (!(RD_REG(cntrl, LINE_STATUS_REG) & LSR_XMIT_EMPTY)) {
			/* */
		}
  		/* baud rate divisor */
  		/* we are using 9600 baud as default. */
  		WR_REG(cntrl, LINE_CNTRL_REG, LCR_DLAB);
  		WR_REG(cntrl, DIVISOR_LATCH_MSB_REG, 0);
  		WR_REG(cntrl, DIVISOR_LATCH_LSB_REG, 24);
		
  		/* set operating parameters and set DLAB to 0 */
  		WR_REG(cntrl, LINE_CNTRL_REG, LCR_8_BITS_CHAR | LCR_1_STOP_BITS);

  		/* no interrupt */
  		WR_REG(cntrl, INTR_ENABLE_REG, 0x0);
		
  		/* disable FIFO mode and reset both FIFOs */
#if 1
  		WR_REG(cntrl, FIFO_CNTRL_REG, FCR_DISABLE_FIFO);
#else
  		/* enable FIFO mode and reset both FIFOs */
  		WR_REG(cntrl, FIFO_CNTRL_REG, FCR_ENABLE_FIFO);
  		WR_REG(cntrl, FIFO_CNTRL_REG,
         		FCR_ENABLE_FIFO | FCR_RCV_FIFO_RESET | FCR_RCV_FIFO_RESET);

  		/* flush all pending input */
  		/* #define    CIRC_FLUSH(x)
           		((x)->db_in = (x)->db_out = (x)->db_buf) */
  		CIRC_FLUSH(&ns16550ports[unit].si_ibuf);
#endif
    		s403_ready = 1;
    		s403_print("\n403GCX serial port ready.\n");
	}
}

int
s403_poll(void) {
	register volatile unsigned char   *cntrl = S403_BASE;
	
	if (s403_ready < 0)
		return -1;
	if (!s403_ready)
		s403_init();
	if (RD_REG(cntrl, LINE_STATUS_REG) & LSR_DATA_READY)
		return RD_REG(cntrl, RCV_BUF_REG);
	else
		return -1;
		
}

int
s403_getchar() {
	register volatile unsigned char   *cntrl = S403_BASE;
	char data;
	int status;

	if (s403_ready < 0)
		return 0;
	if (!s403_ready)
		s403_init();

	status = RD_REG(cntrl, LINE_STATUS_REG);

	while (!((RD_REG(cntrl, LINE_STATUS_REG) & LSR_DATA_READY)));
	data = RD_REG(cntrl, RCV_BUF_REG);

	/* Clear any error conditions, yes we'll ignore error for now.  */
	if (status & (LSR_OVERRUN_ERR | LSR_PARITY_ERR | LSR_FRAMING_ERR)) {
#if 0
	     printf("ERROR: EXAR_UART_16C2550, status = 0x%x\n", status);
#endif
	     return 0;
	}
	return data & 0x000000ff;
}

void
s403_putchar(int c) {
	register volatile unsigned char   *cntrl = S403_BASE;

	if (s403_ready < 0)
		return;
	if (!s403_ready)
		s403_init();
	while(!(RD_REG(cntrl, LINE_STATUS_REG) & LSR_XMIT_BUF_EMPTY));
	WR_REG(cntrl, XMIT_BUF_REG, (unsigned int)c);
}

void
s403_print(const char *c) {
	if (s403_ready < 0)
		return -1;
        while (*c) {
		if (*c == '\n')
			s403_putchar('\r');
		s403_putchar(*c);
		c++;
	}
}

int
s403_readline(char *c, int max) {
	int i = 0;

	if (s403_ready < 0)
		return 0;
	while (i < max) {
		c[i] = s403_getchar();
		if (c[i] == '\n') {
			c[i] = '\0';
			return i;
		}
		i++;
	}
	return i;
}

#else
/*
 * Really, really simple serial port driver for the IBM 403G series
 * PowerPC processor with onboard serial port.
 *
 * Copyright (C) 1998 Teleworld, Inc. All Rights Reserved.
 */

#include	<asm/io.h>
#include	<asm/ppcops.h>


static unsigned char *S403_BASE = (unsigned char *) IOBASE_SERIAL;

#define S403_STATUS	(S403_BASE+0)	/* status register */
#define S403_HANDSHAKE	(S403_BASE+2)	/* handshake status */
#define S403_BAUDHIGH	(S403_BASE+4)	/* baud rate generator */
#define S403_BAUDLOW	(S403_BASE+5)
#define S403_CONTROL	(S403_BASE+6)
#define S403_RX		(S403_BASE+7)	/* receiver command */
#define S403_TX		(S403_BASE+8)	/* transmitter command */
#define S403_RDWR	(S403_BASE+9)	/* send/receive buffer */

void s403_init(void);
int s403_getchar(void);
void s403_putchar(int);
void s403_print(const char *);
int s403_readline(char *, int);

char	s403_ready = 0;

void
s403_virtualinit(void) {
	s403_ready = 0;
	S403_BASE = (unsigned char *) IOBASE_SERIAL;
	s403_init();
}

void
s403_realinit(void) {
	s403_ready = 0;
	S403_BASE = (unsigned char *) TCD_IO_BASE_SERIAL;
	s403_init();
}

#if 0
void
s403_rx(unsigned int irq, void *dev_id, struct pt_regs *regs) {
#ifdef CONFIG_XMON
	int c = inb(S403_RDWR);
	if (c != '')
		return;
	xmon(regs);
#endif
}
#endif

void
s403_init() {
	if (!s403_ready) {
	    unsigned long iocr = mfdcr(SPRN_IOCR);
#ifdef CONFIG_TCD_EVAL
	    iocr |= 0x3;
#else
	    iocr &= ~0x3;
	    iocr |= 0x1;
#endif
	    mtdcr(SPRN_IOCR, iocr);

	    outb(0x18, S403_CONTROL);
	    outb(0, S403_HANDSHAKE);
	    outb(0x78, S403_STATUS);
	    outb(0, S403_BAUDHIGH);
#ifdef CONFIG_TCD_EVAL
	    outb(0x2f, S403_BAUDLOW);
#else
	    outb(0xae, S403_BAUDLOW);
#endif
	    outb(0x88, S403_RX);
	    outb(0x80, S403_TX);

#if 0
	    request_irq(IRQ_403SERIAL_RX, s403_rx, 0, "403serial", 0);
#endif
	    s403_ready = 1;
	    s403_print("\n403GCX serial port ready.\n");
	}
}

int
s403_poll(void) {
	if (!s403_ready)
		s403_init();
	if (inb(S403_STATUS) & 0x80) 
	    return inb(S403_RDWR);
	else
	    return -1;
}

int
s403_getchar() {
	int stat;
	if (!s403_ready)
		s403_init();
	do {
	    stat = inb(S403_STATUS);
	} while (!(stat & 0x80));
	return inb(S403_RDWR);
}

void
s403_putchar(int c) {
	int stat;
	if (!s403_ready)
		s403_init();
	do {
	    stat = inb(S403_STATUS);
	} while (!(stat & 0x4));
	outb(c, S403_RDWR);
}

void
s403_print(const char *c) {
	unsigned char *d = (unsigned char *)c;
	unsigned int stat;
	int sawCR = 0;
	if (!s403_ready)
		s403_init();
	while (*d) {
	    do {
		stat = inb(S403_STATUS);
	    } while (!(stat & 0x4));
	    if (*d == '\n' && !sawCR) {
	    	outb('\r', S403_RDWR);
	    	sawCR = 1;
	    	continue;
	    }
	    if (*d == '\r')
	    	sawCR = 1;
	    outb(*d++, S403_RDWR);
	}
}

int
s403_readline(char *c, int max) {
	int i = 0;
	int stat;
	if (!s403_ready)
		s403_init();
	while (i < max) {
	    do {
		stat = inb(S403_STATUS);
	    } while (!(stat & 0x80));
	    c[i] = inb(S403_RDWR);
	    if (c[i] == '\n') {
	    	c[i] = '\0';
	    	return i;
	    }
	    i++;
	}
	return i;
}
#endif
