/*
 * Driver for PowerMac version of Qlogic PCI SCSI controller (ISP1020)
 *
 */
#ifndef _QLOGIC_PMAC_H
#define _QLOGIC_PMAC_H

/*
 * With the qlogic interface, every queue slot can hold a SCSI
 * command with up to 4 scatter/gather entries.  If we need more
 * than 4 entries, continuation entries can be used that hold
 * another 7 entries each.  Unlike for other drivers, this means
 * that the maximum number of scatter/gather entries we can
 * support at any given time is a function of the number of queue
 * slots available.  That is, host->can_queue and host->sg_tablesize
 * are dynamic and _not_ independent.  This all works fine because
 * requests are queued serially and the scatter/gather limit is
 * determined for each queue request anew.
 */

#define QLOGICISP_REQ_QUEUE_LEN	63	/* must be power of two - 1 */
#define QLOGICISP_MAX_SG(x)     (4 + ((x) > 0) ? 7*((x) - 1) : 0)

extern struct proc_dir_entry proc_scsi_qlc;

/* Prototypes ****************************************************************/
int qlc_detect( Scsi_Host_Template * );
int qlc_release( struct Scsi_Host * );
const char * qlc_info( struct Scsi_Host * );
int qlc_queue( Scsi_Cmnd *, void (*done)(Scsi_Cmnd *) );
int qlc_abort( Scsi_Cmnd * );
int qlc_reset( Scsi_Cmnd *, unsigned int );
int qlc_bios_param( Disk *, kdev_t, int[] );
void qlc_print_scsi_cmd( Scsi_Cmnd * );
/* End Prototypes ************************************************************/

#define SCSI_QLOGIC_PMAC {				\
        NULL,			/* next */		\
	NULL,			/* usage_count */	\
	&proc_scsi_qlc,         /* proc_dir */		\
	NULL,			/* proc_info */		\
	"QLC",			/* name */		\
	qlc_detect,		/* detect */		\
	qlc_release,		/* release */		\
	qlc_info,		/* info */		\
	NULL,   		/* command */		\
	qlc_queue,		/* queuecommand */	\
	qlc_abort,		/* abort */		\
	qlc_reset,		/* reset */		\
	NULL,			/* slave_attach */	\
	qlc_bios_param,		/* bios_param */	\
	QLOGICISP_REQ_QUEUE_LEN,/* can_queue */		\
	-1,			/* this_id */		\
	SG_ALL,			/* sg_tablesize ?? */	\
	1,			/* cmd_per_lun */	\
	0,			/* present */		\
	0,			/* unchecked_isa_dma */	\
	DISABLE_CLUSTERING,	/* use_clustering */	\
}

#endif
