/* TiVo:

   Block device geometry and timing specifications.  These are custom
   additions to the Linux Kernel to support realtime I/O.

*/

#define SECTOR_BYTES 512

typedef struct FsIovec {
    void *pb;
    unsigned int cb;
} FsIovec;

enum FsIovecMode { IOVEC_READ, IOVEC_WRITE }; 

struct FsIovecState {
    struct FsIovec *head;
    int head_len;
    enum FsIovecMode mode; /* read or write */

    struct FsIovec *curr_iovec; /* the currently "active" iovec */
    int curr_iovec_bytes; /* number of bytes into the "active" iovec */
    int total_bytes; /* total number of bytes so far */
};

struct FsDiskState {
    int c; /* cylinder */
    int h; /* head */
    int s; /* sector */
}; 

void FsIovecWrite(char *src,
                  int src_len,
                  struct FsIovec *dest);

void FsIovecInit(struct FsIovecState *state,
                 struct FsIovec *head,
                 int head_len,
                 enum FsIovecMode mode);
int FsIovecTransfer(struct FsIovecState *state,
                    char *ptr, int bytes);
#if 0
struct FsDiskZone {
    /* information calculated&used at runtime. */
    int nStartSector;
    int nEndSector;
    int nStartTrack;
    int nEndTrack;

    /* information used to generate tables. */
    unsigned char nZoneNumber;
    int nTracks;
    int nSectorsPerTrack;
    /* kilobytes/second for this zone: */
    unsigned int bandwidth; 
};

struct FsDiskGeometry {
    /* is this information valid? (set by CalculateZones, or not at all) */
    int valid; 

    int nHeads;
    int nCylinders;
    int nBytesPerSector;
    int rpm;

    /* timing information: (all numbers should be worst case) */

    /* track-to-track seek time in usec */
    unsigned int seekSingle;
    /* full stroke seek time in usec */
    unsigned int seekFull;
    /* head switch time in usec */
    unsigned int headSwitch;
    /* rotational latency */
    unsigned int rotationalLatency; 

    struct FsDiskZone vZones[16];
};    

void CalculateZones(struct FsDiskGeometry *thiz);
void TranslateToLogical (struct FsDiskGeometry *thiz,
                int nCylinder, int nHead, int nSector,
                int *pLogicalSector);

void TranslateToCHS(struct FsDiskGeometry *thiz,
               int nLogicalSector,
               int *pCylinder, int *pHead, int *pSector);

int GetLogicalSectorMax(struct FsDiskGeometry *thiz);
const struct FsDiskZone *GetZone(
    struct FsDiskGeometry *thiz,
    int nTrack);
struct request; 
unsigned long FsDiskCalcTime(struct request *rq,
                             struct FsDiskState *pos,
                             struct FsDiskGeometry *disk); 

#endif /* 0 */
